@@ -30,6 +30,7 @@ predict.dgp2vec <- function(object, x_new, m = object$m, ordering_new = NULL,
                             lite = TRUE, store_latent = FALSE, mean_map = TRUE, 
                             return_all = FALSE, EI = FALSE, entropy_limit = NULL,
                             cores = 1, ...) {
+
   settings <- list(ordering_new = ordering_new, lite = lite,
                    store_latent = store_latent, mean_map = mean_map,
                    return_all = return_all, EI = EI, 
@@ -61,17 +62,26 @@ predict_vec <- function(object, x_new, m, settings, layers) {
   
   tic <- proc.time()[[3]]
   if (is.numeric(x_new)) x_new <- as.matrix(x_new)
+  monowarp <- (!is.null(object$x_grid))
   object$x_new <- x_new
   n_new <- nrow(object$x_new)
   object$m_pred <- m
+
+  if (monowarp) {
+    grid_index <- fo_approx_init(object$x_grid, object$x)
+    grid_index_new <- fo_approx_init(object$x_grid, x_new)
+  }
+
   if (layers >= 2) {
-    D <- ncol(object$w[[1]]) # dimension of latent layer(s)
+    if (monowarp) D <- ncol(object$w_grid[[1]]) else D <- ncol(object$w[[1]])
     if (!settings$mean_map) 
       stop("mean_map = FALSE is not available for `vecchia = TRUE` case")
   }
+
   if (layers == 1) {
     sep <- is.matrix(object$theta)
-  } else sep <- FALSE # two and three layers never use separable lengthscales
+  } else sep <- monowarp # separable lengthscales only used in monotonic warpings
+  
   if (settings$return_all & !settings$lite) 
     stop("return_all only offered when lite = TRUE")
   if (!is.null(settings$entropy_limit) & !is.numeric(settings$entropy_limit))
@@ -87,13 +97,15 @@ predict_vec <- function(object, x_new, m, settings, layers) {
     test <- check_ordering(settings$ordering_new, nrow(x_new))
   }
   
-  # Pre-calculate nearest neighbors for x 
-  if (settings$lite) {
-    NN_x_new <- FNN::get.knnx(object$x, x_new, m)$nn.index
-    x_approx <-  NULL
-  } else { 
-    NN_x_new <- NULL
-    x_approx <- add_pred_to_approx(object$x_approx, x_new, m, settings$ordering_new)
+  # If not monotonic, pre-calculate nearest neighbors for x
+  if (!monowarp) {
+    if (settings$lite) {
+      NN_x_new <- FNN::get.knnx(object$x, x_new, m)$nn.index
+      x_approx <-  NULL
+    } else { 
+      NN_x_new <- NULL
+      x_approx <- add_pred_to_approx(object$x_approx, x_new, m, settings$ordering_new)
+    }
   }
   
   # Initialize prior mean of 0 (these will only be changed if object$settings$pmx = TRUE)
@@ -135,41 +147,60 @@ predict_vec <- function(object, x_new, m, settings, layers) {
       if (layers >= 2) {
         # 2 layers: map x_new to w_new (separately for each dimension)
         # 3 layers: map z_new to w_new (separately for each dimension)
-        w_t <- object$w[[t]]
-        w_new <- matrix(nrow = n_new, ncol = D)
-        for (i in 1:D) {
-          if (layers == 2) {
-            if (object$settings$pmx) { # Optional prior mean of x
-              prior_mean_new <- x_new[, i]
-              prior_mean <- object$x[, i]
-              prior_tau2 <- object$settings$inner_tau2
-            }
-          } 
-          k <- krig_vec(w_t[, i], object$theta_w[t, i], g = eps, tau2 = prior_tau2,
-                        v = object$v, m = m, x = ifel(layers == 2, object$x, z_t), 
-                        x_new = ifel(layers == 2, x_new, z_new),
-                        NNarray_pred = ifel(layers == 2, NN_x_new, NULL),
-                        prior_mean = prior_mean, 
-                        prior_mean_new = prior_mean_new) # lite = TRUE version only
-          w_new[, i] <- k$mean
-        } # end of i for loop
-        if (settings$store_latent) w_new_list[[t]] <- w_new
-        
-        if (settings$lite) {
-          w_approx <- NULL
+        if (monowarp) { # simply get the monowarped values at x_new locations
+          w_grid_t <- object$w_grid[[t]]
+          w_t <- monowarp_ref(object$x, object$x_grid, w_grid_t, grid_index)
+          w_new <- monowarp_ref(x_new, object$x_grid, w_grid_t, grid_index_new)
+          if (settings$lite) {
+            w_approx <- NULL
+          } else {
+            w_approx <- update_obs_in_approx(object$w_approx, w_t)
+            w_approx <- add_pred_to_approx(w_approx, w_new, m, settings$ordering_new)
+          }
         } else {
-          w_approx <- update_obs_in_approx(object$w_approx, w_t)
-          w_approx <- add_pred_to_approx(w_approx, w_new, m, settings$ordering_new)
-        }
+          w_t <- object$w[[t]]
+          w_new <- matrix(nrow = n_new, ncol = D)
+          for (i in 1:D) {
+            if (layers == 2) {
+              if (object$settings$pmx) { # Optional prior mean of x
+                prior_mean_new <- x_new[, i]
+                prior_mean <- object$x[, i]
+                prior_tau2 <- object$settings$inner_tau2
+              }
+            } 
+            k <- krig_vec(w_t[, i], object$theta_w[t, i], g = eps, tau2 = prior_tau2,
+                          v = object$v, m = m, x = ifel(layers == 2, object$x, z_t), 
+                          x_new = ifel(layers == 2, x_new, z_new),
+                          NNarray_pred = ifel(layers == 2, NN_x_new, NULL),
+                          prior_mean = prior_mean, 
+                          prior_mean_new = prior_mean_new) # lite = TRUE version only
+            w_new[, i] <- k$mean
+          } # end of i for loop
+          if (settings$store_latent) w_new_list[[t]] <- w_new
+          
+          if (settings$lite) {
+            w_approx <- NULL
+          } else {
+            w_approx <- update_obs_in_approx(object$w_approx, w_t)
+            w_approx <- add_pred_to_approx(w_approx, w_new, m, settings$ordering_new)
+          }
+        } # end of monowarp else statement
       }
       
       # 1 layer: map x_new to y
       # 2 and 3 layers: map w_new to y
-      if (sep) { # only occurs in one layer
-        k <- krig_vec(object$y, object$theta[t, ], object$g[t], object$tau2[t],
+      if (sep) { # only occurs in one layer or monowarped two-layer
+        k <- krig_vec(object$y, 
+                      ifel(monowarp, object$theta_y[t, ], object$theta[t, ]), 
+                      object$g[t], 
+                      object$tau2[t],
                       s2 = settings$lite, sigma = !settings$lite, v = object$v, 
-                      m = m, x = object$x, x_new = x_new, NNarray_pred = NN_x_new,
-                      approx = x_approx, sep = TRUE, f_min = f_min)
+                      m = m, 
+                      x = ifel(monowarp, w_t, object$x), 
+                      x_new = ifel(monowarp, w_new, x_new), 
+                      NNarray_pred = ifel(monowarp, NULL, NN_x_new),
+                      approx = ifel(monowarp, w_approx, x_approx), 
+                      sep = TRUE, f_min = f_min)
       } else {
         k <- krig_vec(object$y, ifel(layers == 1, object$theta[t], object$theta_y[t]), 
                       object$g[t], object$tau2[t],
@@ -248,41 +279,59 @@ predict_vec <- function(object, x_new, m, settings, layers) {
         if (layers >= 2) {
           # 2 layers: map x_new to w_new (separately for each dimension)
           # 3 layers: map z_new to w_new (separately for each dimension)
-          w_t <- object$w[[t]]
-          w_new <- matrix(nrow = n_new, ncol = D)
-          for (i in 1:D) {
-            if (layers == 2) {
-              if (object$settings$pmx) { # Optional prior mean of x
-                prior_mean_new <- x_new[, i]
-                prior_mean <- object$x[, i]
-                prior_tau2 <- object$settings$inner_tau2
-              }
-            } 
-            k <- krig_vec(w_t[, i], object$theta_w[t, i], g = eps, tau2 = prior_tau2,
-                          v = object$v, m = m, x = ifel(layers == 2, object$x, z_t), 
-                          x_new = ifel(layers == 2, x_new, z_new),
-                          NNarray_pred = ifel(layers == 2, NN_x_new, NULL),
-                          prior_mean = prior_mean, 
-                          prior_mean_new = prior_mean_new) # lite = TRUE version only
-            w_new[, i] <- k$mean
-          } # end of i for loop
-          if (settings$store_latent) out$w_new[[j]] <- w_new
-          
-          if (settings$lite) {
-            w_approx <- NULL
+          if (monowarp) { # simply get the monowarped values at x_new locations
+            w_grid_t <- object$w_grid[[t]]
+            w_t <- monowarp_ref(object$x, object$x_grid, w_grid_t, grid_index)
+            w_new <- monowarp_ref(x_new, object$x_grid, w_grid_t, grid_index_new)
+            if (settings$lite) {
+              w_approx <- NULL
+            } else {
+             w_approx <- update_obs_in_approx(object$w_approx, w_t)
+              w_approx <- add_pred_to_approx(w_approx, w_new, m, settings$ordering_new) 
+            }
           } else {
-            w_approx <- update_obs_in_approx(object$w_approx, w_t)
-            w_approx <- add_pred_to_approx(w_approx, w_new, m, settings$ordering_new)
-          }
+            w_t <- object$w[[t]]
+            w_new <- matrix(nrow = n_new, ncol = D)
+            for (i in 1:D) {
+              if (layers == 2) {
+                if (object$settings$pmx) { # Optional prior mean of x
+                  prior_mean_new <- x_new[, i]
+                  prior_mean <- object$x[, i]
+                  prior_tau2 <- object$settings$inner_tau2
+                }
+              } 
+              k <- krig_vec(w_t[, i], object$theta_w[t, i], g = eps, tau2 = prior_tau2,
+                            v = object$v, m = m, x = ifel(layers == 2, object$x, z_t), 
+                            x_new = ifel(layers == 2, x_new, z_new),
+                            NNarray_pred = ifel(layers == 2, NN_x_new, NULL),
+                            prior_mean = prior_mean, 
+                            prior_mean_new = prior_mean_new) # lite = TRUE version only
+              w_new[, i] <- k$mean
+            } # end of i for loop
+            if (settings$store_latent) out$w_new[[j]] <- w_new
+            
+            if (settings$lite) {
+              w_approx <- NULL
+            } else {
+              w_approx <- update_obs_in_approx(object$w_approx, w_t)
+              w_approx <- add_pred_to_approx(w_approx, w_new, m, settings$ordering_new)
+            }
+          } # end of monowarp else statement
         }
         
         # 1 layer: map x_new to y
         # 2 and 3 layers: map w_new to y
-        if (sep) { # only occurs in one layer
-          k <- krig_vec(object$y, object$theta[t, ], object$g[t], object$tau2[t],
+        if (sep) { # only occurs in one layer or monowarped two-layer
+          k <- krig_vec(object$y, 
+                        ifel(monowarp, object$theta_y[t, ], object$theta[t, ]), 
+                        object$g[t], object$tau2[t],
                         s2 = settings$lite, sigma = !settings$lite, v = object$v, 
-                        m = m, x = object$x, x_new = x_new, NNarray_pred = NN_x_new,
-                        approx = x_approx, sep = TRUE, f_min = f_min)
+                        m = m, 
+                        x = ifel(monowarp, w_t, object$x), 
+                        x_new = ifel(monowarp, w_new, x_new), 
+                        NNarray_pred = ifel(monowarp, NULL, NN_x_new),
+                        approx = ifel(monowarp, w_approx, x_approx), 
+                        sep = TRUE, f_min = f_min)
         } else {
           k <- krig_vec(object$y, ifel(layers == 1, object$theta[t], object$theta_y[t]), 
                         object$g[t], object$tau2[t],
